/***************************************************************************
                            FLTester.h
                        -------------------
begin                : lun 29 Mar 2004
copyright            : (C) 2004-2005 by InfoSiAL S.L.
email                : mail@infosial.com
***************************************************************************/
/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License.               *
 ***************************************************************************/
/***************************************************************************
   Este  programa es software libre. Puede redistribuirlo y/o modificarlo
   bajo  los  trminos  de  la  Licencia  Pblica General de GNU   en  su
   versin 2, publicada  por  la  Free  Software Foundation.
 ***************************************************************************/

#ifndef FLTESTER_H
#define FLTESTER_H

#include <qapplication.h>
#include <qdom.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qobjectlist.h>
#include <qtoolbutton.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpixmap.h>
#include <qbuffer.h>
#include <qwidgetfactory.h>
#include <qsqlquery.h>
#include <qsqlcursor.h>
#include <qtimer.h>
#include <qradiobutton.h>
#include <qeventloop.h>
#include <qvaluelist.h>
#include <qgroupbox.h>
#include <qptrqueue.h>
#include <qtabwidget.h>

class FLFormDB;
class FLSqlDatabase;

class FLTestFdb {

public:
  QString fdbName;
  QString fdbValue;
};

/**
Eventos para las pruebas
*/
class FL_EXPORT FLTesterEvent {

public:

  /**
  constructor
  */
  FLTesterEvent() {}

  /**
  destructor
  */
  ~FLTesterEvent() {}

  /**
  Condicin actual que debe cumplirse para continuar con el guin de pruebas
  */
  int condition;

  /**
  Nombre del objeto sobre el que se aplica la condicin actual
  */
  QString objectCondition;
};

/**
Clase para automatizar las pruebas de calidad del software

@author InfoSiAL S.L.
*/
class FL_EXPORT FLTester: public QObject {

  Q_OBJECT

public:

  /**
  constructor.
  */
  FLTester();

  /**
  destructor
  */
  virtual ~FLTester() ;

  /**
  Constantes para indicar el resultado al ejecutar un paso
  */
  enum StepResult {
    /** El paso se complet correctamente */
    OK = 0,
    /** Hubo un error controlado en la ejecucin del paso */
    ERR = 1,
    /** El paso debe repetirse por no estar listo todava el entorno */
    REPEAT = 2,
    /** Hubo un error de ejecucin en el paso */
    TESTER_ERROR = 3,
    /** La ejecucin del guin debe saltarse la prueba actual*/
    SKIP_TEST = 4,
    /** La ejecucin del guin debe continuar sin esperar*/
    NEXT = 5
  };

  /**
  Constantes para indicar la condicin que debe cumplirse para continuar ejecutando el guin de pruebas
  */
  enum Condition {
    /** Cualquier hay condicin */
    NO_CONDITION,
    /** FLFormDB listo */
    FORM_READY,
    /** FLFormDB cerrado */
    FORM_CLOSED,
    /** FLFormRecordDB listo */
    FORM_RECORD_READY,
    /** FLFormSearchDB listo */
    FORM_SEARCH_READY,
    /** FLFieldDB listo */
    FIELD_READY,
    /** MessageBox listo */
    MESSAGE_BOX_READY,
    /** MessageBox cerrado */
    MESSAGE_BOX_CLOSED,
    /** Registro borrado */
    RECORD_DELETED,
    /** Script finalizado*/
    SCRIPT_FINISHED,
    /** Ninguna condicin*/
    NONE_IS_VALID
  };


  /**
  Obtiene la lista de sesiones cargadas en la base de datos de test
  @param bd Nombre de la base de datos de test
  @return Cadena que contiene la lista con formato idfuncional//codsessioncat//description**...
  */
  QString sessionsList( const QString &bd );

  /**
  Devuelve el identificador actual de la sesin de ejecucin de tests
  */
  int currentSession() {
    return currentSession_;
  }

  /**
  Devuelve el estado de la sesin actual
  */
  QString sessionStatus() {
    return sessionStatus_;
  }

  /**
  Devuelve la conexin a la base de datos utilizada para los tests
  */
  FLSqlDatabase * dbTest() {
    return dbTest_;
  }

  /**
  Obtiene si se estn ejecutando tests
  */
  bool testing() {
    return testing_;
  }

  /**
  Encola un evento en la cola de eventos para las pruebas

  @param c: Condicin
  @param oC: Objeto sobre el que se impone la condicin
  */
  void enqueueTesterEvent( const int c, const QString & oC );

public slots:

  /**
  Inicia sesin de tests

  @param codSessionCat: Cdigo de la sesin a lanzar
  @param idFuncional: Identificador de la funcionalidad a la que pertenece la sesin
  @param bd: Nombre de la base de datos que contiene el catlogo de pruebas
  */
  void start( const QString &codSessionCat, const QString &idFuncional = QString::null, const QString &bd = QString::null );

protected:

  /**
  Crea los pasos correspondientes a una sesin de pruebas en el guin de pasos

  @param codSessionCat: Cdigo de la sesin a lanzar
  @param idFuncional: Identificador de la funcionalidad a la que pertenece la sesin
  @return: Identificador de la sesin creada, 0 si hay error
  */
  uint createSession( const QString &codSessionCat, const QString &idFuncional = QString::null );

  /**
  Aade los pasos de un test al guin de pasos

  @param idTest; Identificador del test en el guin de pasos
  @param idTestCat; Identificador del test en el catlogo de pasos
  @param testLevel; Indicador del nivel de anidamiento de pruebas
  @return: True si no hay error, false en caso contrario
  */
  bool addTest( const uint &idTest, const QString idTestCat, uint testLevel );

  /**
  Abre la base de datos de tests, y la guarda en dbTest_

  @param bd: Nombre de la base de datos
  */
  void openTestDB( const QString &bd );

protected slots:

  /**
  Termina la sesin de tests.
  */
  void end();

  /**
  Ejecuta un paso del guin de pasos

  @return: True si el paso se complet correctamente y hay ms pasos que procesar
  */
  bool execTest();

  /**
  Intenta ejecutar la siguiente prueba del guin de pruebas. La prueba se ejecutar si la condicin y el objeto indicados son los esperados

  @param c: Condicin
  @param oC: Objeto sobre el que se impone la condicin
  @return True la prueba se puede ejecutar y se ejecuta correctamente, false en caso contrario
  */
  bool tryContinue( const int c, const QString & oC );

  /**
  Esencialmente hace lo mismo que el mtodo FLTester::tryContinue( const int c, const QString & oC ).

  En este mtodo obtiene la informacin de condicin y objeto de la cola de eventos, ver FLTesterEvent y queueTesterEvents
  */
  bool tryContinue();

private:

  /**
  Lanza un paso del guin de pasos

  @param idStepCat: Identificador del paso en el catlogo de pasos
  @param idStep: Identificador del paso en el guin de pruebas
  @return (valor de tipo StepResult)
  */
  int execStep( const QString & idStepCat, const QString & idStep );

  /**
  Cambia el estado de la prueba actual

  @param status: estado de la prueba
  @param errorDescription: Descripcin del error produducido, en su caso
  */
  void setTestStatus( const QString errorDescription );

  /**
  Cambia el estado de la sesin actual

  @param status: estado de la prueba
  */
  void setSessionStatus( const QString );

  /**
  Obtiene el valor resultado de una consulta

  @param n: Nodo XML con los datos de la consulta
  @param v: Variable donde se devolver el resutado de la consulta (FALSE si la consulta no encuentra registros o el primer valor encontrado)
  @return Cdigo de error similar al de execStep
  */
  int getQueryValue( QDomNode n, QVariant &v );

  /**
  Obtiene el valor resultado asociado a un nodo de comparacin

  @param n: Nodo XML con los datos del elemento a comparar
  @param v: Variable donde se devolver el valor del elemento
  @return Cdigo de error similar al de execStep
  */
  int getValue( QDomNode n, QVariant &v );

  /**
  Preprocesa el string de parmetros sustituyendo las variables por sus valores

  @param parameters: String de parmetros a procesar
  @return True si el preproceso se completa con xito, false en caso contrario
  */
  bool processParameters( QString &parameters );

  /**
  Establece la condicin requerida para continuar con el siguiente paso del guin de pruebas

  @param e: Nodo que contiene los atributos que definen la condicin
  @param defCondition: Condicin por defecto
  @param defObjectCondition: Objeto asociado a la condicin por defecto
  */
  void setConditions( QDomElement e, const Condition & defCondition, const QString & defObjectCondition );

  /**
  Realiza una transaccin a travs de la conexin de la base de datos de pruebas

  @param qrySql: Sentencia SQL a ejecutar
  @return True si la transaccin se completa correctamente (se realiza el Commit), false en caso contrario (Rollback)
  */
  bool execTransaction( const QString &qrySql );

  /**
  Asigna valores a variables globales

  @param parameters: Nodo XML con las asignaciones a realizar
  @return True si la asignacin se completa correctamente, false en caso contrario 
  */
  bool assignateValues( const QString &parameters );

  /**
  Asigna valores por defecto necesarios para ejecutar una determinada prueba

  @param testName: Nodo XML con los identificadores (identificador y funcionalidad) de la prueba
  @return True si la asignacin se completa correctamente, false en caso contrario 
  */
  bool assignateDefaultValues( const QString & testName );

  /**
  Asigna un valor a una variable global, usando la conexin de pruebas

  @param name: Nombre de la variable
  @param value: Valor de la variable
  @return True si la asignacin se completa correctamente, false en caso contrario 
  */
  bool setVar( const QString &name, const QVariant &value );

  /**
  Obtiene el valor de una variable global, usando la conexin de pruebas

  @param name: Nombre de la variable
  @param exists: Indicador de si la variable est o no definida
  @return Valor de la variable
  */
  QVariant getVar( const QString &name, bool *exists );

  /**
  Elimina una variable global, usando la conexin de pruebas

  @param name: Nombre de la variable
  @return True si la eliminacin se completa correctamente, false en caso contrario 
  */
  bool delVar( const QString &name );

  /**
  Nmero de fallos tolerados hasta que se para la prueba
  */
  int fallos_;

  /**
  Nmero correspondiente a la prueba anterior
  */
  QString previousTest_;

  /**
  Nmero correspondiente a la prueba actual
  */
  QString currentTest_;

  /**
  Nmero correspondiente a la sesin actual
  */
  int currentSession_;

  /**
  Formulario actual sobre el que se realiza la prueba
  */
  FLFormDB *currentForm_;

  /**
  Conexin a la base de datos de test.
  */
  FLSqlDatabase * dbTest_;

  /**
  Nombre del mdulo que se est probando
  */
  QString currentModule_;

  /**
  Estado de la sesin de pruebas actual
  */
  QString sessionStatus_;

  /**
  Descripcin del error
  */
  QString errorDescription_;

  /**
  Indica si se estn ejecutando tests
  */
  bool testing_;

  /**
  Condicin actual que debe cumplirse para continuar con el guin de pruebas
  */
  int condition_;

  /**
  Nombre del objeto sobre el que se aplica la condicin actual
  */
  QString objectCondition_;

  /**
  Cola de eventos
  */
  QPtrQueue < FLTesterEvent > * queueTesterEvents;

  /**
  Lista de campos FLFieldDB que pueden ser modificados en un nico paso
  */
  QPtrQueue < FLTestFdb > * fdbQueue_;
};

class FL_EXPORT FLEventLoop: public QEventLoop {

  Q_OBJECT

public:

  FLEventLoop();
  virtual bool processEvents( ProcessEventsFlags flags );
  virtual int enterLoop();
  virtual void exitLoop();

private:
  bool aboutToEnter_;
};

#endif
